import importlib
from fastapi import FastAPI
from fastapi.responses import JSONResponse
from typing import Optional, Union
from .constructors import ValidYTSearchRequest

class YT(FastAPI):
    """
    YT Endpoints
    """   
    def __init__(self, app: FastAPI, util,
                 constants) -> None:
        self.app: FastAPI = app
        self.util = util
        self.constants = constants
        self.ytsearch = importlib.import_module("youtube_search_async").YoutubeSearch()

        self.endpoints: dict = {
            "yt/search": self.yt_video_search_handler,
            }

        for endpoint, handler in self.endpoints.items():
            app.add_api_route(f"/{endpoint}", handler, methods=["POST"],
                              include_in_schema=True)
    
    async def yt_video_search_handler(self, data: ValidYTSearchRequest) -> JSONResponse:
        """
        Search for YT Video by Title (closest match returned)
        - **t**: Title to search
        """
        
        title: str = data.t
        yts_res: Optional[list[dict]] = await self.ytsearch.search(title)
        if not yts_res:
            return JSONResponse(status_code=404, content={
                'err': True,
                'errorText': 'No result.',
            })
        yt_video_id: Union[str, bool] = yts_res[0].get('id', False)

        return JSONResponse(content={
            'video_id': yt_video_id,
            'extras': yts_res[0],
        })